@@ -1,7 +1,8 @@
 /*
  * Copyright (c) 2003-2006 Erez Zadok
  * Copyright (c) 2003-2006 Charles P. Wright
- * Copyright (c) 2005-2006 Josef Sipek
+ * Copyright (c) 2005-2006 Josef 'Jeff' Sipek
+ * Copyright (c) 2005-2006 Junjiro Okajima
  * Copyright (c) 2005      Arun M. Krishnakumar
  * Copyright (c) 2005-2006 David P. Quigley
  * Copyright (c) 2003-2004 Mohammad Nayyer Zubair
@@ -16,7 +17,7 @@
  * This Copyright notice must be kept intact and distributed with all sources.
  */
 /*
- *  $Id: inode.c,v 1.271 2006/06/01 03:11:02 jsipek Exp $
+ *  $Id: inode.c,v 1.275 2006/10/31 18:05:33 yiannos Exp $
  */
 
 #include "unionfs.h"
@@ -474,8 +475,7 @@ static int unionfs_mkdir(struct inode *parent, struct dentry *dentry, int mode)
 	int bindex = 0, bstart;
 	char *name = NULL;
 	int whiteout_unlinked = 0;
-	uid_t saved_uid = current->fsuid;
-	gid_t saved_gid = current->fsgid;
+	struct sioq_args args;
 
 	print_entry_location();
 	lock_dentry(dentry);
@@ -505,21 +505,15 @@ static int unionfs_mkdir(struct inode *parent, struct dentry *dentry, int mode)
 	} else {
 		hidden_parent_dentry = lock_parent(whiteout_dentry);
 
-		/* Set the uid and gid to trick the fs into allowing us to create
-		 * the file */
-		current->fsuid = hidden_parent_dentry->d_inode->i_uid;
-		current->fsgid = hidden_parent_dentry->d_inode->i_gid;
 		//found a.wh.foo entry, remove it then do vfs_mkdir
 		if (!(err = is_robranch_super(dentry->d_sb, bstart))) {
-			err =
-			    vfs_unlink(hidden_parent_dentry->d_inode,
-				       whiteout_dentry);
+			args.unlink.parent = hidden_parent_dentry->d_inode;
+			args.unlink.dentry = whiteout_dentry;
+			run_sioq(__unionfs_unlink, &args);
+			err = args.err;
 		}
 		DPUT(whiteout_dentry);
 
-		current->fsuid = saved_uid;
-		current->fsgid = saved_gid;
-
 		unlock_dir(hidden_parent_dentry);
 
 		if (err) {
@@ -531,7 +525,7 @@ static int unionfs_mkdir(struct inode *parent, struct dentry *dentry, int mode)
 			whiteout_unlinked = 1;
 		}
 	}
-	
+
 	for (bindex = bstart; bindex >= 0; bindex--) {
 		hidden_dentry = dtohd_index(dentry, bindex);
 		if (!hidden_dentry) {
@@ -564,7 +558,7 @@ static int unionfs_mkdir(struct inode *parent, struct dentry *dentry, int mode)
 		} else {
 			int i;
 			int bend = dbend(dentry);
-			
+
 			for (i = bindex + 1; i < bend; i++) {
 				if (dtohd_index(dentry, i)) {
 					DPUT(dtohd_index(dentry, i));
@@ -830,7 +824,7 @@ static int inode_permission(struct inode *inode, int mask, struct nameidata *nd,
 	} else {
 		retval = generic_permission(inode, submask, NULL);
 	}
-	
+
 	if (retval && retval != -EROFS) /* ignore EROFS */
 		return retval;
 
@@ -865,7 +859,7 @@ static int unionfs_permission(struct inode *inode, int mask,
 		 */
 		if (!is_file && !S_ISDIR(hidden_inode->i_mode))
 			continue;
-		/* We use our own special version of permission, such that 
+		/* We use our own special version of permission, such that
 		 * only the first branch returns -EROFS. */
 		err = inode_permission(hidden_inode, mask, nd, bindex);
 		/* The permissions are an intersection of the overall directory
@@ -917,7 +911,7 @@ static int unionfs_setattr(struct dentry *dentry, struct iattr *ia)
 				continue;
 			/* Only if its the leftmost file, copyup the file */
 			for (i = bstart - 1; i >= 0; i--) {
-				size_t size = dentry->d_inode->i_size;
+				loff_t size = dentry->d_inode->i_size;
 				if (ia->ia_valid & ATTR_SIZE)
 					size = ia->ia_size;
 				err = copyup_dentry(dentry->d_parent->d_inode,
@@ -941,7 +935,7 @@ static int unionfs_setattr(struct dentry *dentry, struct iattr *ia)
 		break;
 	}
 #ifdef UNIONFS_MMAP
-	/* 
+	/*
 	 * SP: notify_change will change the lower file's size,
 	 * but we need to truncate the page tables, so need to call
 	 * vmtruncate()
