@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2003-2005 Erez Zadok
  * Copyright (c) 2003-2006 Erez Zadok
  * Copyright (c) 2003-2006 Charles P. Wright
- * Copyright (c) 2005-2006 Josef Sipek
+ * Copyright (c) 2005-2006 Josef 'Jeff' Sipek
+ * Copyright (c) 2005-2006 Junjiro Okajima
  * Copyright (c) 2005      Arun M. Krishnakumar
  * Copyright (c) 2005-2006 David P. Quigley
  * Copyright (c) 2003-2004 Mohammad Nayyer Zubair
@@ -17,7 +17,7 @@
  * This Copyright notice must be kept intact and distributed with all sources.
  */
 /*
- *  $Id: super.c,v 1.96 2006/06/25 23:57:43 jro Exp $
+ *  $Id: super.c,v 1.101 2006/11/04 22:27:51 jsipek Exp $
  */
 
 #include "unionfs.h"
@@ -158,14 +158,17 @@ static void unionfs_put_super(struct super_block *sb)
 	print_exit_location();
 }
 
-static int unionfs_statfs(struct super_block *sb, struct kstatfs *buf)
+static int unionfs_statfs(struct dentry *dentry, struct kstatfs *buf)
 {
 	int err = 0;
-	struct super_block *hidden_sb;
+	struct super_block *sb, *hidden_sb;
 	struct kstatfs rsb;
 	int bindex, bindex1, bstart, bend;
 
 	print_entry_location();
+
+	sb = dentry->d_sb;
+
 	memset(buf, 0, sizeof(struct kstatfs));
 	buf->f_type = UNIONFS_SUPER_MAGIC;
 
@@ -190,7 +193,7 @@ static int unionfs_statfs(struct super_block *sb, struct kstatfs *buf)
 			continue;
 		}
 
-		err = vfs_statfs(hidden_sb, &rsb);
+		err = vfs_statfs(hidden_sb->s_root, &rsb);
 		dprint(PRINT_DEBUG,
 		       "adding values for bindex:%d bsize:%d blocks:%d bfree:%d bavail:%d\n",
 		       bindex, (int)rsb.f_bsize, (int)rsb.f_blocks,
@@ -268,7 +271,8 @@ static int unionfs_remount_fs(struct super_block *sb, int *flags, char *data)
 {
 	if (data && *((unsigned long *)data) == UNIONFS_REMOUNT_MAGIC)
 		return do_binary_remount(sb, flags, data);
-	return -ENOSYS;
+	printk("Warning! dirs delete and imap options to remount are ignored\n");
+	return 0;
 }
 
 /*
@@ -411,8 +415,6 @@ static int unionfs_write_inode(struct inode *inode, int sync)
 }
 
 /*
- * Called in do_umount() if the MNT_FORCE flag was used and this
- * function is defined.  See comment in linux/fs/super.c:do_umount().
  * Used only in nfs, to kill any pending RPC tasks, so that subsequent
  * code can actually succeed and won't leave tasks that need handling.
  *
@@ -420,21 +422,34 @@ static int unionfs_write_inode(struct inode *inode, int sync)
  * left in the kernel after a user level file server (such as amd)
  * dies.
  */
-static void unionfs_umount_begin(struct super_block *sb)
+static void unionfs_umount_begin(struct vfsmount *mnt, int flags)
 {
-	struct super_block *hidden_sb;
+	struct super_block *sb, *hidden_sb;
+	struct vfsmount *hidden_mnt;
 	int bindex, bstart, bend;
 
 	print_entry_location();
 
+	if (!(flags & MNT_FORCE))
+		/* we are not being MNT_FORCEd, therefore we should emulate old
+		 * behaviour
+		 */
+		goto out;
+
+	sb = mnt->mnt_sb;
+
 	bstart = sbstart(sb);
 	bend = sbend(sb);
 	for (bindex = bstart; bindex <= bend; bindex++) {
-		hidden_sb = stohs_index(sb, bindex);
-		if (hidden_sb && hidden_sb->s_op &&
+		hidden_mnt = stohiddenmnt_index(sb, bindex);
+		hidden_sb  = stohs_index(sb, bindex);
+
+		if (hidden_mnt && hidden_sb && hidden_sb->s_op &&
 		    hidden_sb->s_op->umount_begin)
-			hidden_sb->s_op->umount_begin(hidden_sb);
+			hidden_sb->s_op->umount_begin(hidden_mnt, flags);
 	}
+
+out:
 	print_exit_location();
 }
 
@@ -670,18 +685,22 @@ enum {
 static void do_decode(__u32 * fh, struct dentry **hidden_root,
 		      ino_t * hidden_ino, ino_t * hidden_parent_ino)
 {
-	*hidden_root = (void *)fh[FhHRoot2];
+	unsigned long root;
+	
+	root = fh[FhHRoot2];
 	*hidden_ino = fh[FhHIno2];
 	*hidden_parent_ino = fh[FhHPIno2];
 #if BITS_PER_LONG == 64
-	*hidden_root |= fh[FhHRoot1] << 32;
-	*hidden_ino |= fh[FhHIno1] << 32;
-	*hidden_parent_ino |= fh[FhHPIno1] << 32;
+	root |= ((unsigned long)fh[FhHRoot1]) << 32;
+	*hidden_ino |= ((unsigned long) fh[FhHIno1]) << 32;
+	*hidden_parent_ino |= ((unsigned long) fh[FhHPIno1]) << 32;
 #elif BITS_PER_LONG == 32
 	/* ok */
 #else
 #error unknown size
 #endif
+
+	*hidden_root = (struct dentry*) root;
 }
 
 static int unionfs_encode_fh(struct dentry *dentry, __u32 * fh, int *max_len,
@@ -725,11 +744,11 @@ static int unionfs_encode_fh(struct dentry *dentry, __u32 * fh, int *max_len,
 	unlock_dentry(sb->s_root);
 	h_ino = itohi_index(dentry->d_inode, bindex)->i_ino;
 	hp_ino = parent_ino(dtohd(dentry));
-	fh[FhHRoot2] = (__u32) h_root;
+	fh[FhHRoot2] = (unsigned long) h_root;
 	fh[FhHIno2] = h_ino;
 	fh[FhHPIno2] = hp_ino;
 #if BITS_PER_LONG == 64
-	fh[FhHRoot1] = h_root >> 32;
+	fh[FhHRoot1] = ((unsigned long) h_root) >> 32;
 	fh[FhHIno1] = h_ino >> 32;
 	fh[FhHPIno1] = hp_ino >> 32;
 #endif
