@@ -202,6 +202,9 @@ struct unionfs_dir_state {
 	struct list_head uds_list[0];
 };
 
+/* privileged io workqueue */
+#include "sioq.h"
+
 /* include miscellaneous macros */
 #include "unionfs_macros.h"
 
@@ -387,7 +390,7 @@ extern int unionfs_getlk(struct file *file, struct file_lock *fl);
 extern int unionfs_file_revalidate(struct file *file, int willwrite);
 extern int unionfs_open(struct inode *inode, struct file *file);
 extern int unionfs_file_release(struct inode *inode, struct file *file);
-extern int unionfs_flush(struct file *file);
+extern int unionfs_flush(struct file *file, fl_owner_t id);
 extern long unionfs_ioctl(struct file *file, unsigned int cmd,
 			  unsigned long arg);
 
@@ -687,14 +690,6 @@ static inline void unlock_dir(struct dentry *dir)
 	DPUT(dir);
 }
 
-struct superio {
-	uid_t fsuid;
-	struct __user_cap_data_struct cap;
-	unsigned long rlim_core;
-};
-extern void superio_store(struct superio *sio);
-extern void superio_revert(struct superio *sio);
-
 extern int make_dir_opaque(struct dentry *dir, int bindex);
 
 #endif				/* __KERNEL__ */
